using System;
using System.Collections.Generic;
using System.Text;

namespace EnDatLatchAcquisition
{
    class MSXE173x
    {
        /// <summary>
        /// IP address of the system
        /// </summary>
        private String ip;

        private MSXE173xWeb.MSXE17xx webRef;
        private MXCommonWeb.MXCommon commonwebRef;

        /// <summary>
        /// Accessor on the IP
        /// </summary>
        public String Ip
        {
            get
            {
                return this.ip;
            }
            set
            {
                // Create common interface
                this.commonwebRef = new MXCommonWeb.MXCommon();
                this.commonwebRef.Url = "http://" + value + ":5555";
                this.commonwebRef.Timeout = 2000;

                // try a request that will throw an exception if the IP is not correct
                {
                    byte[] tmp = null;
                    this.commonwebRef.GetHostname(out tmp);
                }

                // Good, create the specific interface
                this.webRef = new MSXE173xWeb.MSXE17xx();
                this.webRef.Url = "http://" + value + ":5555";
                this.webRef.Timeout = 2000;

                // Copy the ip
                this.ip = value;
            }
        }

        /// <summary>
        /// Initialises an EnDat sensor
        /// </summary>
        /// <param name="connectorIndex"></param>
        /// <param name="frequency"></param>
        /// <param name="error"></param>
        /// <returns></returns>
        public int EndatInitSensor(ulong connectorIndex, ulong frequency, out string error)
        {
            int returnValue = 0;
            int syserrno = 0;

            // Initialize output values
            error = string.Empty;

            // Call the soap function
            try
            {
                returnValue = this.webRef.MFEndatInitSensor(connectorIndex,
                    0,
                    frequency,
                    0,
                    0,
                    0,
                    0,
                    out syserrno);

                if (returnValue != 0)
                {
                    error = "MFEndatInitSensor exception : " + returnValue + " syserrno : " + syserrno;
                    return returnValue;
                }
            }
            catch (Exception ex)
            {
                error = "MFEndatInitSensor exception : " + ex.Message;
                return -2;
            }

            // success
            return 0;
        }

        /// <summary>
        /// Initialise the latch acquisition
        /// </summary>
        /// <param name="connectorIndex"></param>
        /// <param name="latchSource"></param>
        /// <param name="triggerEdgeCount"></param>
        /// <param name="dataFormat"></param>
        /// <param name="error"></param>
        /// <returns></returns>
        public int EndatInitAndEnableLatchPositionValues(ulong connectorIndex,
            ulong latchSource,
            ulong triggerEdgeCount,
            ulong dataFormat,
            out string error)
        {
            int returnValue = 0;
            int syserrno = 0;

            error = "";

            // Call the soap function
            try
            {
                returnValue = this.webRef.MFEndatInitAndEnableLatchPositionValues(connectorIndex,
                    0,
                    latchSource,
                    triggerEdgeCount,
                    dataFormat,
                    0,
                    0,
                    0,
                    0,
                    out syserrno);


                if (returnValue != 0)
                {
                    error = "EndatInitAndEnableLatchPositionValues exception : " + returnValue + " syserrno : " + syserrno;
                    return -1;

                }
            }
            catch (Exception ex)
            {
                error = "EndatInitAndEnableLatchPositionValues exception : " + ex.Message;
                return -2;
            }
            return 0;
        }

        /// <summary>
        /// Disable and release the latch logic
        /// </summary>
        /// <param name="moduleIndex"></param>
        /// <param name="channelIndex"></param>
        /// <param name="error"></param>
        /// <returns></returns>
        public int EndatDisableAndReleaseLatchPositionValues(ulong connectorIndex,
            out string error)
        {
            int returnValue = 0;
            int syserrno = 0;
            error = "";

            // Call the soap function
            try
            {
                returnValue = this.webRef.MFEndatDisableAndReleaseLatchPositionValues(connectorIndex,
                    0,
                    0,
                    0,
                    0,
                    0,
                    out syserrno);

                if ((returnValue != 0) || (syserrno != 0))
                {
                    error = "EndatDisableAndReleaseLatchPositionValues exception : " + returnValue + " syserrno : " + syserrno;
                    return -1;

                }
            }
            catch (Exception ex)
            {
                error = "EndatDisableAndReleaseLatchPositionValues exception : " + ex.Message;
                return -2;
            }
            return 0;
        }

        /// <summary>
        /// Reads the properties of the EnDat sensor
        /// </summary>
        /// <param name="connectorIndex"></param>
        /// <param name="IDNumberLsb"></param>
        /// <param name="IDNumberMsb"></param>
        /// <param name="serialNumberLsb"></param>
        /// <param name="serialNumberMsb"></param>
        /// <param name="model"></param>
        /// <param name="mode"></param>
        /// <param name="positionSize"></param>
        /// <param name="signalPeriod"></param>
        /// <param name="stepPerRevolution"></param>
        /// <param name="numberOfRevolution"></param>
        /// <param name="scalingFactor"></param>
        /// <param name="additionalData"></param>
        /// <param name="error"></param>
        /// <returns></returns>
        public int EndatGetSensorProperties(ulong connectorIndex,
            out ulong IDNumberLsb,
            out ulong IDNumberMsb,
            out ulong serialNumberLsb,
            out ulong serialNumberMsb,
            out ulong model,
            out ulong mode,
            out ulong positionSize,
            out ulong signalPeriod,
            out ulong stepPerRevolution,
            out ulong numberOfRevolution,
            out ulong scalingFactor,
            out ulong additionalData,
            out string error)
        {
            MSXE173xWeb.DefaultResponse response = null;

            IDNumberLsb = 0;
            IDNumberMsb = 0;
            serialNumberLsb = 0;
            serialNumberMsb = 0;
            model = 0;
            mode = 0;
            positionSize = 0;
            signalPeriod = 0;
            stepPerRevolution = 0;
            numberOfRevolution = 0;
            scalingFactor = 0;
            additionalData = 0;
            error = string.Empty;

            // Call the soap function
            try
            {
                response = this.webRef.MFEndatGetSensorProperties(connectorIndex,
                  0,
                  0,
                  0,
                  0,
                  0,
                  out IDNumberLsb,
                  out IDNumberMsb,
                  out serialNumberLsb,
                  out serialNumberMsb,
                  out model,
                  out mode,
                  out positionSize,
                  out signalPeriod,
                  out stepPerRevolution,
                  out numberOfRevolution,
                  out scalingFactor,
                  out additionalData);

                if ((response.iReturnValue != 0) || (response.syserrno != 0))
                {
                    error = "MFEndatGetEncoderModel exception : " + response.iReturnValue + " syserrno : " + response.syserrno;
                    return -1;
                }
            }
            catch (Exception ex)
            {
                error = "MFEndatGetEncoderModel exception : " + ex.Message;
                return -2;
            }
            return 0;
        }

        /// <summary>
        /// Returns the size in byte of the packet send by the dataserver for the selected connector
        /// </summary>
        /// <param name="dataFormat"></param>
        /// <param name="connectorIndex"></param>
        /// <param name="size"></param>
        /// <param name="error"></param>
        /// <returns></returns>
        public int EndatGetPacketSize(ulong dataFormat, ulong connectorIndex, out int size, out string error)
        {
            ulong IDNumberLsb = 0;
            ulong IDNumberMsb = 0;
            ulong serialNumberLsb = 0;
            ulong serialNumberMsb = 0;
            ulong model = 0;
            ulong mode = 0;
            ulong positionSize = 0;
            ulong signalPeriod = 0;
            ulong stepPerRevolution = 0;
            ulong numberOfRevolution = 0;
            ulong scalingFactor = 0;
            ulong additionalData = 0;
            error = string.Empty;
            int retval = 0;

            // initialisation
            size = 8; // event source and error are always there

            // time stamp ?
            if ((dataFormat & 1) == 1)
                size += 8;

            // digital I/O
            if ((dataFormat & 2) == 2)
                size += 4;

            // additional data 1
            if ((dataFormat & 4) == 4)
                size += 4;

            // additional data 2
            if ((dataFormat & 8) == 8)
                size += 4;

            // If not standardised, then 2 uint32 for the position
            if ((dataFormat & 16) != 16)
            {
                size += 8;
                return 0;
            }

            // If standardised, we need the properties
            // Get the properties
            retval = this.EndatGetSensorProperties(connectorIndex, 
                out IDNumberLsb, 
                out IDNumberMsb, 
                out serialNumberLsb, 
                out serialNumberMsb, 
                out model, 
                out mode, 
                out positionSize, 
                out signalPeriod, 
                out stepPerRevolution, 
                out numberOfRevolution, 
                out scalingFactor, 
                out additionalData, 
                out error);

            if (retval != 0)
                return retval;

            // absolute linear encoder
            if ((model == 4) || (model == 6))
            {
                // position is one float
                size += 4;
                return 0;
            }

            if ((model == 13) || (model == 14))
            {
                // position is two uint32 and one float for the angle position
                size += 12;
                return 0;
            }

            // default two uint32
            size += 8;

            return 0;
        }

        /// <summary>
        /// Reads the model of the sensor
        /// </summary>
        /// <param name="connectorIndex"></param>
        /// <param name="model"></param>
        /// <param name="error"></param>
        /// <returns></returns>
        public int EndatGetSensorModel(ulong connectorIndex, out ulong model, out string error)
        {
            ulong IDNumberLsb = 0;
            ulong IDNumberMsb = 0;
            ulong serialNumberLsb = 0;
            ulong serialNumberMsb = 0;
            ulong mode = 0;
            ulong positionSize = 0;
            ulong signalPeriod = 0;
            ulong stepPerRevolution = 0;
            ulong numberOfRevolution = 0;
            ulong scalingFactor = 0;
            ulong additionalData = 0;
            error = string.Empty;
            int retval = 0;

            // Get the properties
            retval = this.EndatGetSensorProperties(connectorIndex,
                out IDNumberLsb,
                out IDNumberMsb,
                out serialNumberLsb,
                out serialNumberMsb,
                out model,
                out mode,
                out positionSize,
                out signalPeriod,
                out stepPerRevolution,
                out numberOfRevolution,
                out scalingFactor,
                out additionalData,
                out error);

            return retval;
        }
    }
}
