using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace ErrorMonitor
{
    public partial class MainForm : Form
    {
        /// <summary>
        /// Store the old error state
        /// </summary>
        private ulong[] olderrorstate;

        /// <summary>
        /// Useful variables
        /// </summary>
        private Panel[][] errorPanels;
        private bool first = true;

        /// <summary>
        /// Constructor of the form
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            // Create useful graphic variable
            this.errorPanels = new Panel[4][];
            this.errorPanels[0] = new Panel[11];
            this.errorPanels[0][0] = this.channelError00;
            this.errorPanels[0][1] = this.channelError01;
            this.errorPanels[0][2] = this.channelError02;
            this.errorPanels[0][3] = this.channelError03;
            this.errorPanels[0][4] = this.channelError04;
            this.errorPanels[0][5] = this.channelError05;
            this.errorPanels[0][6] = this.channelError06;
            this.errorPanels[0][7] = this.channelError07;
            this.errorPanels[0][8] = this.channelError08;
            this.errorPanels[0][9] = this.channelError09;
            this.errorPanels[0][10] = this.channelError010;

            this.errorPanels[1] = new Panel[11];
            this.errorPanels[1][0] = this.channelError10;
            this.errorPanels[1][1] = this.channelError11;
            this.errorPanels[1][2] = this.channelError12;
            this.errorPanels[1][3] = this.channelError13;
            this.errorPanels[1][4] = this.channelError14;
            this.errorPanels[1][5] = this.channelError15;
            this.errorPanels[1][6] = this.channelError16;
            this.errorPanels[1][7] = this.channelError17;
            this.errorPanels[1][8] = this.channelError18;
            this.errorPanels[1][9] = this.channelError19;
            this.errorPanels[1][10] = this.channelError110;

            this.errorPanels[2] = new Panel[11];
            this.errorPanels[2][0] = this.channelError20;
            this.errorPanels[2][1] = this.channelError21;
            this.errorPanels[2][2] = this.channelError22;
            this.errorPanels[2][3] = this.channelError23;
            this.errorPanels[2][4] = this.channelError24;
            this.errorPanels[2][5] = this.channelError25;
            this.errorPanels[2][6] = this.channelError26;
            this.errorPanels[2][7] = this.channelError27;
            this.errorPanels[2][8] = this.channelError28;
            this.errorPanels[2][9] = this.channelError29;
            this.errorPanels[2][10] = this.channelError210;

            this.errorPanels[3] = new Panel[11];
            this.errorPanels[3][0] = this.channelError30;
            this.errorPanels[3][1] = this.channelError31;
            this.errorPanels[3][2] = this.channelError32;
            this.errorPanels[3][3] = this.channelError33;
            this.errorPanels[3][4] = this.channelError34;
            this.errorPanels[3][5] = this.channelError35;
            this.errorPanels[3][6] = this.channelError36;
            this.errorPanels[3][7] = this.channelError37;
            this.errorPanels[3][8] = this.channelError38;
            this.errorPanels[3][9] = this.channelError39;
            this.errorPanels[3][10] = this.channelError310;
        }

        /// <summary>
        /// Occurs when the form is loaded
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_Load(object sender, EventArgs e)
        {
            this.ipTextBox.Text = Properties.Settings.Default.ip;
        }

        /// <summary>
        /// Do the work
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void updateWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            BackgroundWorker worker = (BackgroundWorker)sender;
            MSXE173xWeb.MSXE17xx sys = new MSXE173xWeb.MSXE17xx();
            sys.Url = "http://" + this.ipTextBox.Text + ":5555";
            sys.Timeout = 2000;

            for (; ; )
            {
                ErrorStatus status = new ErrorStatus();

                try
                {
                    // Get the error source
                    for (int i = 0; i < 4; i++)
                    {
                        ulong errorsrc = 0;
                        MSXE173xWeb.DefaultResponse response = sys.MFEndatGetErrorSources((ulong)i, 0,
                            0, 0, 0, 0,
                            out errorsrc);
                        if (response.iReturnValue != 0)
                        {
                            worker.ReportProgress(0, "MFEndatGetErrorSources error : " + response.iReturnValue.ToString());
                            return;
                        }

                        status.errorstatus[i] = errorsrc;
                    }
                }
                catch (Exception ex)
                {
                    worker.ReportProgress(0, "MFEndatGetErrorSources exception : " + ex.Message);
                    return;
                }

                // Report
                worker.ReportProgress(1, status);

                // Sleep
                System.Threading.Thread.Sleep(500);
            }
        }

        /// <summary>
        /// Occurs when we got new error state
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void updateWorker_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            if (e.ProgressPercentage == 0)
            {
                this.currentStatusLabel.Text = (String)e.UserState;
                this.currentStatusLabel.ForeColor = Color.Red;
            }
            else if (e.ProgressPercentage == 1)
            {
                ErrorStatus status = (ErrorStatus)e.UserState;
                
                // Show new error status
                for (int i = 0; i < status.errorstatus.Length; i++)
                {
                    if ( ( status.errorstatus[i] == this.olderrorstate[i] ) && ( this.first == false ) )
                        continue;

                    if ((status.errorstatus[i] & 1) == 1)
                        this.errorPanels[i][0].BackColor = Color.Red;
                    else
                        this.errorPanels[i][0].BackColor = Color.Chartreuse;

                    if (((status.errorstatus[i] >> 1) & 1) == 1)
                        this.errorPanels[i][1].BackColor = Color.Red;
                    else
                        this.errorPanels[i][1].BackColor = Color.Chartreuse;

                    if (((status.errorstatus[i] >> 2) & 1) == 1)
                        this.errorPanels[i][2].BackColor = Color.Red;
                    else
                        this.errorPanels[i][2].BackColor = Color.Chartreuse;

                    if (((status.errorstatus[i] >> 3) & 1) == 1)
                        this.errorPanels[i][3].BackColor = Color.Red;
                    else
                        this.errorPanels[i][3].BackColor = Color.Chartreuse;

                    if (((status.errorstatus[i] >> 4) & 1) == 1)
                        this.errorPanels[i][4].BackColor = Color.Red;
                    else
                        this.errorPanels[i][4].BackColor = Color.Chartreuse;

                    if (((status.errorstatus[i] >> 6) & 1) == 1)
                        this.errorPanels[i][5].BackColor = Color.Red;
                    else
                        this.errorPanels[i][5].BackColor = Color.Chartreuse;

                    if (((status.errorstatus[i] >> 7) & 1) == 1)
                        this.errorPanels[i][6].BackColor = Color.Red;
                    else
                        this.errorPanels[i][6].BackColor = Color.Chartreuse;

                    if (((status.errorstatus[i] >> 8) & 1) == 1)
                        this.errorPanels[i][7].BackColor = Color.Red;
                    else
                        this.errorPanels[i][7].BackColor = Color.Chartreuse;

                    if (((status.errorstatus[i] >> 9) & 1) == 1)
                        this.errorPanels[i][8].BackColor = Color.Red;
                    else
                        this.errorPanels[i][8].BackColor = Color.Chartreuse;

                    if (((status.errorstatus[i] >> 10) & 1) == 1)
                        this.errorPanels[i][9].BackColor = Color.Red;
                    else
                        this.errorPanels[i][9].BackColor = Color.Chartreuse;

                    if (((status.errorstatus[i] >> 11) & 1) == 1)
                        this.errorPanels[i][10].BackColor = Color.Red;
                    else
                        this.errorPanels[i][10].BackColor = Color.Chartreuse;
                }

                // Copy old status
                for (int i = 0; i < this.olderrorstate.Length; i++)
                {
                    this.olderrorstate[i] = status.errorstatus[i];
                }

                this.first = false;
            }
        }

        /// <summary>
        /// Initialise
        /// </summary>
        private void initState()
        {
            this.olderrorstate = new ulong[4];
            for (int i = 0; i < 4; i++)
            {
                this.olderrorstate[i] = 0;
            }
            this.first = true;

            // Set to white
            for (int i = 0; i < this.errorPanels.Length; i++)
            {
                for (int j = 0; j < this.errorPanels[i].Length; j++)
                {
                    this.errorPanels[i][j].BackColor = Color.White;
                }
            }
        }

        /// <summary>
        /// User click on start
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void startButton_Click(object sender, EventArgs e)
        {
            // Click on the start button
            this.startButton.Enabled = false;
            this.ipTextBox.Enabled = false;

            // Start the progress bar
            this.runningProgressBar.Visible = true;

            // Update state
            this.currentStatusLabel.Text = "Started";
            this.currentStatusLabel.ForeColor = Color.Green;

            // Initialise the state
            this.initState();
            
            // Start the background worker
            this.updateWorker.RunWorkerAsync();
        }

        /// <summary>
        /// Occurs when the form is being closed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            Properties.Settings.Default.ip = this.ipTextBox.Text;
            Properties.Settings.Default.Save();
        }

        /// <summary>
        /// Update worker is completed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void updateWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            this.startButton.Enabled = true;
            this.ipTextBox.Enabled = true;
            this.runningProgressBar.Visible = false;
        }

        /// <summary>
        /// Reset the error bits
        /// </summary>
        private void resetError( ulong index )
        {
            if (startButton.Enabled == true)
                return;

            MSXE173xWeb.MSXE17xx sys = new MSXE173xWeb.MSXE17xx();
            sys.Url = "http://" + this.ipTextBox.Text + ":5555";
            sys.Timeout = 2000;

            try
            {
                int syserrno = 0;
                sys.MFEndatResetErrorBits(index, 0, 0, 0, 0, 0, out syserrno);
            }
            catch (Exception)
            {

            }
        }

        /// <summary>
        /// Click on reset error channel 0
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void resetErrorButton0_Click(object sender, EventArgs e)
        {
            this.resetError(0);
        }

        /// <summary>
        /// Click on reset error channel 1
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void resetErrorButton1_Click(object sender, EventArgs e)
        {
            this.resetError(1);
        }

        /// <summary>
        /// Click on reset error channel 2
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void resetErrorButton2_Click(object sender, EventArgs e)
        {
            this.resetError(2);
        }

        /// <summary>
        /// Click on reset error channel 3
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void resetErrorButton3_Click(object sender, EventArgs e)
        {
            this.resetError(3);
        }
    }
}